
restart;
# This function depends on a global variable POLYS containing a list of polynomials
#	to use in the optimization

cutplane_opt := proc(f, Q0, bds, eps, rounding, temp_decay)
local Q, feas, vars, new_constrs, c_val, m_val, LPres, ti, delete_num, delete_index, r, temp,
	bestsofar, real_max;
	bestsofar:=[0.5,[]];
	Q := Q0;
	feas := false;
	temp := 0.5;
	while (not feas) do
		ti := time();
		LPres := Optimization[LPSolve](f, Q, seq(bds), iterationlimit=100000);
		print(time());
		print("time to solve LP: ", time()-ti);
		vars := LPres[2];
		c_val := LPres[1];
		m_val := [seq(op(2,vars[i]),i=2..nops(vars))];
		print("c_val: ", c_val);
		print("m_val: ", m_val);
		ti:= time();
		new_constrs, real_max := violated_t_SIP([seq(op(2,vars[i]),i=1..nops(vars))],
											eps, rounding);

		if real_max < bestsofar[1] then
			bestsofar := [real_max, m_val];
			print("************************");
			print("************************");
			print("************************");
			print("NEW BEST FOUND");
			print("************************");
			print("************************");
			print("************************");
		end if;
											
		print("time to find new constraints: ", time()-ti);
		ti := time();
		if new_constrs = true then
			feas := true;
		else
			delete_num := temp * nops(Q);
			temp := temp * temp_decay;
			print("temp: ", temp);
			r := rand(1..nops(Q));
			delete_index := [seq(r(),i=1..delete_num)];
			Q := [Q[1],Q[2],seq(`if`(i in delete_index, NULL, Q[i]), i=3..nops(Q))];
			Q := [op(Q), op(new_constrs)];
			print("number of constraints: ", nops(Q));
		end if;
		print("time to add constraints to Q: ", time()-ti);
	end do;
	return vars, fval;
end proc:

violated_t_SIP := proc(cm, eps, rounding)
local c_val, m_val, constrs, poly, vals, v, LHS, RHS, polyvals, local_maxis, real_max;

	c_val := cm[1];
	m_val := seq(cm[i],i=2..nops(cm));

	constrs := [];
	
	poly := mul(POLYS[i]^(round((m_val[i]/degree(POLYS[i],t))*rounding)),i=1..nops(POLYS));
	
	vals := local_maxi(poly);

	local_maxis := [seq(abs(subs(t=v,poly)), v in vals)];

	real_max := max(local_maxis)^(1/degree(poly));
	
	print("real_max", real_max);
	print("exp(c_val): ", exp(c_val));

	if real_max < exp(c_val) + eps then
		print("WITHIN ERROR BOUND");
		return true;
	end if;

	for v in vals do
		
		if abs(subs(t=v,poly))^(1/degree(poly)) > exp(c_val) then
			polyvals := [seq((subs(t=v,POLYS[i]))^2, i=1..nops(POLYS))];
			if min(polyvals) > 0 then
				LHS := add(m[i]*(log(polyvals[i]))/(degree(POLYS[i], t)), i=1..nops(POLYS));
				RHS := 2*c;
				constrs := [op(constrs), LHS <= RHS];
			end if;
		end if;
	end do;

	if nops(constrs) = 0 then
		return true, real_max;
	else
		return constrs, real_max;
	end if;
	
end proc:

local_maxi := proc(p)
local i, j, m, poly, poly2, v, v2, vals;
	if type(p, `+`) then poly := diff(p, t);
	else
		poly := 0;
		for i to nops(p) do
			poly2 := 1;
			for j to nops(p) do
				if i = j then
					if type(op(j, p), `^`) then
						poly2 := poly2 * diff(op(1, op(j, p)), t) * op(2, op(j, p));
					else
						poly2 := poly2 * diff(op(j, p), t);
					end if;
				else
					if type(op(j, p), `^`) then
						poly2 := poly2 * op(1, op(j, p));
					else
						poly2 := poly2 * op(j, p);
					end if;
				end if;
			end do;
			poly := poly + poly2;
		end do;
	end if;
	vals := {0, 0.5, fsolve(poly, t = 0 .. 0.5)};
	return vals;
end proc:

alt_log := proc(x)
	if x <= 0 then
		return float(-infinity);
	else 
		return log(x);
	end if;
end proc:

maxi := proc(p)
local i, j, m, poly, poly2, v, v2, vals, r, polyset;
	if type(p, `+`) then
		poly := diff(p, t);
		polyset := [p];
		r[1] := 1;
	else
		polyset := [seq(`if`(type(op(i, p), `^`), op(1, op(i, p)), op(i, p)), i = 1..nops(p))];
		poly := 0;
		for i to nops(p) do
			
			if type(op(i, p), `^`) then
				r[i] := op(2, op(i, p));
			else
				r[i] := 1;
			end if;
			
			poly2 := 1;
			for j to nops(p) do
				if i = j then
					if type(op(j, p), `^`) then
						poly2 := poly2 * diff(op(1, op(j, p)), t) * op(2, op(j, p));
					else
						poly2 := poly2 * diff(op(j, p), t);
					end if;
				else
					if type(op(j, p), `^`) then
						poly2 := poly2 * op(1, op(j, p));
					else
						poly2 := poly2 * op(j, p);
					end if;
				end if;
			end do;
			poly := poly + poly2;
		end do;
	end if;
	vals := {0.5, fsolve(poly, t = 0 ..0.5)};
	m := 0;
	for v in vals do
		v2 := exp(add(r[i]*log(abs(subs(t = v, polyset[i]))), i = 1..nops(polyset)));
		if m < v2 then m := v2 end if
	end do;
	return m;
end proc:
Digits := 50;
interface(displayprecision = 20);
infolevel[Optimization] := 0;
POLYS_h := [t*(1-t), 2*t-1, 5*t^2-5*t+1, 6*t^2-6*t+1, 29*t^4-58*t^3+40*t^2-11*t+1, 33*t^4-66*t^3+45*t^2-12*t+1, 34*t^4-68*t^3+46*t^2-12*t+1, expand((7*t^2-6*t+1)*(7*t^2-8*t+2)), 161*t^6-483*t^5+575*t^4-345*t^3+109*t^2-17*t+1, expand((13*t^3-19*t^2+8*t-1)*(13*t^3-20*t^2+9*t-1)), 181*t^6-543*t^5+644*t^4-383*t^3+119*t^2-18*t+1, 193*t^6-579*t^5+683*t^4-401*t^3+122*t^2-18*t+1, 821*t^8-3284*t^7+5555*t^6-5171*t^5+2886*t^4-985*t^3+200*t^2-22*t+1, 941*t^8-3764*t^7+6349*t^6-5873*t^5+3243*t^4-1089*t^3+216*t^2-23*t+1, expand((31*t^4-61*t^3+41*t^2-11*t+1)*(31*t^4-63*t^3+44*t^2-12*t+1)), 4921*t^10-24605*t^9+53804*t^8-67586*t^7+53866*t^6-28388*t^5+9995*t^4-2317*t^3+338*t^2-28*t+1, 31169*t^12-187014*t^11+502099*t^10-796200*t^9+828936*t^8-595698*t^7+302334*t^6-108945*t^5+27600*t^4-4783*t^3+537*t^2-35*t+1, 43609*t^12-261654*t^11+704777*t^10-1125390*t^9+1184854*t^8-865270*t^7+448776*t^6-166327*t^5+43659*t^4-7905*t^3+936*t^2-65*t+2, 161429*t^14-1130003*t^13+3599830*t^12-6908941*t^11+8913112*t^10-8165339*t^9+5470288*t^8-2718775*t^7+1005970*t^6-275399*t^5+54846*t^4-7697*t^3+719*t^2-40*t+1, 161813*t^14-1132691*t^13+3608246*t^12-6924493*t^11+8931952*t^10-8181043*t^9+5479474*t^8-2722543*t^7+1007031*t^6-275594*t^5+54867*t^4-7698*t^3+719*t^2-40*t+1, 161929*t^14-1133503*t^13+3610755*t^12-6928991*t^11+8937122*t^10-8185014*t^9+5481532*t^8-2723251*t^7+1007185*t^6-275613*t^5+54868*t^4-7698*t^3+719*t^2-40*t+1, 887981*t^16-7103848*t^15+26189139*t^14-59006633*t^13+90856296*t^12-101276631*t^11+84454852*t^10-53688009*t^9+26265936*t^8-9911593*t^7+2872148*t^6-631701*t^5+103263*t^4-12115*t^3+961*t^2-46*t+1, 907201*t^16-7257608*t^15+26750188*t^14-60243176*t^13+92693614*t^12-103221560*t^11+85965780*t^10-54562008*t^9+26643715*t^8-10032840*t^7+2900545*t^6-636399*t^5+103781*t^4-12149*t^3+962*t^2-46*t+1];
gwpoly := proc (n) if 1 < n then return expand(gwpoly(n-1)^2+gwpoly(n-1)*gwpoly(n-2)^2-gwpoly(n-2)^4) elif n = 1 then return 5*t^2-5*t+1 else return 2*t-1 end if end proc;
POLYS_q := [[seq(gwpoly(i), i = 0 .. 5)][6]];
POLYS_r := [30689*t^12-184134*t^11+494483*t^10-784520*t^9+817442*t^8-588122*t^7+298951*t^6-107936*t^5+27408*t^4-4762*t^3+536*t^2-35*t+1, 179213*t^14-1254491*t^13+3994456*t^12-7658353*t^11+9863413*t^10-9014198*t^9+6019243*t^8-2978749*t^7+1096048*t^6-297935*t^5+58801*t^4-8158*t^3+751*t^2-41*t+1, 890381*t^16-7123048*t^15+26258899*t^14-59158953*t^13+91079142*t^12-101507147*t^11+84628171*t^10-53783834*t^9+26304860*t^8-9923055*t^7+2874524*t^6-632029*t^5+103290*t^4-12116*t^3+961*t^2-46*t+1, 5287361*t^18-47586249*t^17+199139568*t^16-514494900*t^15+918909397*t^14-1203880783*t^13+1198032039*t^12-925178633*t^11+561591777*t^10-269728267*t^9+102663046*t^8-30867601*t^7+7271422*t^6-1322862*t^5+181649*t^4-18155*t^3+1243*t^2-52*t+1];
POLYS_qs := [t^3-6*t^2+9*t-3, t^4-8*t^3+16*t^2-9*t+1, t^5-9*t^4+26*t^3-29*t^2+11*t-1, t^5-9*t^4+27*t^3-31*t^2+12*t-1, t^5-9*t^4+27*t^3-32*t^2+13*t-1, t^6-11*t^5+42*t^4-68*t^3+46*t^2-12*t+1, t^6-11*t^5+42*t^4-68*t^3+47*t^2-13*t+1, t^6-11*t^5+43*t^4-72*t^3+51*t^2-14*t+1, t^7-13*t^6+61*t^5-131*t^4+136*t^3-66*t^2+14*t-1, t^7-13*t^6+61*t^5-133*t^4+142*t^3-71*t^2+15*t-1, t^7-13*t^6+62*t^5-135*t^4+140*t^3-67*t^2+14*t-1, t^7-13*t^6+62*t^5-136*t^4+144*t^3-71*t^2+15*t-1, t^7-13*t^6+62*t^5-137*t^4+146*t^3-72*t^2+15*t-1, t^7-13*t^6+62*t^5-137*t^4+147*t^3-73*t^2+15*t-1, t^7-13*t^6+62*t^5-137*t^4+148*t^3-75*t^2+16*t-1, t^7-13*t^6+62*t^5-138*t^4+150*t^3-76*t^2+16*t-1, t^7-13*t^6+63*t^5-143*t^4+157*t^3-78*t^2+16*t-1, t^7-13*t^6+63*t^5-143*t^4+158*t^3-80*t^2+16*t-1, t^7-13*t^6+63*t^5-143*t^4+159*t^3-82*t^2+17*t-1, t^7-13*t^6+63*t^5-144*t^4+160*t^3-80*t^2+16*t-1, t^8-15*t^7+83*t^6-220*t^5+303*t^4-220*t^3+83*t^2-15*t+1, t^8-15*t^7+84*t^6-228*t^5+323*t^4-240*t^3+91*t^2-16*t+1, t^8-15*t^7+85*t^6-233*t^5+333*t^4-250*t^3+96*t^2-17*t+1, t^8-15*t^7+86*t^6-241*t^5+353*t^4-270*t^3+104*t^2-18*t+1, t^10-18*t^9+134*t^8-538*t^7+1273*t^6-1822*t^5+1560*t^4-766*t^3+200*t^2-24*t+1, t^10-18*t^9+135*t^8-549*t^7+1320*t^6-1920*t^5+1662*t^4-813*t^3+206*t^2-24*t+1, t^12-22*t^11+204*t^10-1050*t^9+3322*t^8-6752*t^7+8944*t^6-7677*t^5+4177*t^4-1388*t^3+265*t^2-26*t+1, t^6-10*t^5+35*t^4-52*t^3+33*t^2-9*t+1];
POLYS_s := [seq(simplify((t*(1-t))^degree(POLYS_qs[i], t)*subs(t = (1-4*t*(1-t))/(t*(1-t)), POLYS_qs[i])), i = 1 .. nops(POLYS_qs))];
POLYS_v := [expand((414157*t^15-3082633*t^14+10506317*t^13-21730488*t^12+30475127*t^11-30666643*t^10+22852753*t^9-12829629*t^8+5465583*t^7-1765190*t^6+428258*t^5-76576*t^4+9759*t^3-836*t^2+43*t-1)*subs(t = 1-t, 414157*t^15-3082633*t^14+10506317*t^13-21730488*t^12+30475127*t^11-30666643*t^10+22852753*t^9-12829629*t^8+5465583*t^7-1765190*t^6+428258*t^5-76576*t^4+9759*t^3-836*t^2+43*t-1))];
u := t*(1-t)/(1-3*t*(1-t));
POLYS_t := [(3*t^2-3*t+1)^14*simplify(subs(t = u, POLYS_s[9]))];
core_POLYS_h := [seq(POLYS_h[i], i in [1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 15, 16, 17, 21, 22])];
core_POLYS_q := [seq(POLYS_q[i], i in [1])];
core_POLYS_r := [seq(POLYS_r[i], i in [1, 2, 3, 4])];
core_POLYS_s := [seq(POLYS_s[i], i in [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28])];
core_POLYS_t := [seq(POLYS_t[i], i in [1])];
core_POLYS_v := [seq(POLYS_v[i], i in [1])];
POLYS_new := [920401*t^16-7363208*t^15+27136468*t^14-61099136*t^13+93976607*t^12-104596838*t^11+87052281*t^10-55204178*t^9+26928800*t^8-10127338*t^7+2923538*t^6-640375*t^5+104242*t^4-12181*t^3+963*t^2-46*t+1];
POLYS := [op(POLYS_h), op(POLYS_q), op(POLYS_r), op(POLYS_s), op(POLYS_t), op(POLYS_v)];
for p in POLYS_new do if has(POLYS, p) then print(p) end if end do;
POLYS := [op(POLYS_h), op(POLYS_q), op(POLYS_r), op(POLYS_s), op(POLYS_t), op(POLYS_v), op(POLYS_new)];
nops(POLYS);
bds := [seq(m[i] = 0 .. 1, i = 2 .. nops(POLYS)), m[1] = 0.1e-9 .. 1];
Q0 := [c >= -.9, add(m[i], i = 1 .. nops(POLYS)) = 1];
ti := time();
SIPres := cutplane_opt(c, Q0, bds, 10^(-15), 10^10, .93);
tracelast;
time()-ti;
c_val := op(2, SIPres[1, 1]);

m_val := seq(op(2, SIPres[1, i]), i = 2 .. nops(SIPres[1]));

m_int := [seq(round(m_val[i]*10^10/degree(POLYS[i])), i = 1 .. nops(POLYS))];

poly := mul(POLYS[i]^m_int[i], i = 1 .. nops(POLYS));

Digits := 60;
a := maxi(poly)^(1/degree(poly, t));

